/* (c) Matt Clark Feb 2003
 *
 * You are free to copy, change and distribute this software BUT
 * you cannot charge for it (even if you change it).
 *
 * The use of this code in your code conveys no rights over
 * your code.  So, if you wish to charge for a program that uses
 * this library that is fine.  You will have to distribute the
 * library free of charge.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Matt.Clark@Nottingham.ac.uk
 */
#include <stdio.h>
#include <stdlib.h>
#include <ncurses.h>

#include "pi_user.h"

// stuff for the rotating symbol
char *moving="\\|/-";
int  mp=0;

// function prototypes here
void init_curses(void);
void print_moving(void);
void print_status(int);
void print_cmd(int,int,char *);
void print_instructions(void);
void print_async_event(int);
// defs
#define ENTER	10
#define ESCAPE	27
// location of bits of the screen
#define MOVING_LINE	19
#define CMD_LINE	22
#define ASYNC_LINE	20
#define ASYNC_COL	5
// command defs
#define IDLE 	0
#define AXIS	1
#define MOVE	2
#define VEL	3
#define MOVEREL	5
#define TOGGLE_MOTOR	6

int main(void){
BOOL ok;

int key,axis,i,pos,sense;
int time,ax,mode=IDLE,current_axis=0,num_index=0;
char number[100];

number[0]='\0';
// init the ncurses screen
	init_curses();
	bkgd(COLOR_PAIR(1));
	move(0,0);
	printw("Matt's PI stage demo 0.32 13 Feb 2003");
	move(1,0);
	printw("About to auto init everything and switch to screen mode");
	refresh();
	move(3,0);
	printw("Opening the board");
	ok=pi_openboard();  // already calls init board
	if(ok==FALSE){
		move(4,0);
		printw("Failed (is the device driver installed?)");
		move(5,0);
		printw("Press any key to quit");
		getch();
		endwin();
		exit(1);
		}
	else{
		move(3,20);
		printw("Ok");
		}
	axis=pi_axis_installed();
	move(4,0);
	printw("The number of axis supported %d",axis);
	for(i=0;i<axis;i++){
		pi_init_axis(i);
		// this is a test program
		sense=pi_auto_set_limit(i);
		move(5,18*i);
		printw("Axis %d: sense %d",i,sense);
		}
	pi_enable_async_capture((void (*)(int))print_async_event);
	print_instructions();
	time=6;
	ax=0;
	halfdelay(1);
	erase();
	// enable event reporting for axis 0-3
	pi_setQMC(PI_SET_INTERRUPT_MASK,0,0x0001);
	pi_setQMC(PI_SET_INTERRUPT_MASK,1,0x0001);
	pi_setQMC(PI_SET_INTERRUPT_MASK,2,0x0001);
	pi_setQMC(PI_SET_INTERRUPT_MASK,3,0x0001);
	do{
		//update the status slowly
		print_status(ax);
		ax++;
		if(ax>=axis)ax=0;
			time=0;
		//print_async_event();
		print_moving();
		print_cmd(mode,current_axis,number);
		refresh();
		key=getch();
		time++;
		switch(key){
			case 'C':
				mode=IDLE;
				move(CMD_LINE,20);printw("*wait*");refresh();
				pi_find_home(current_axis);
				move(CMD_LINE,20);printw("      ");refresh();
				break;
			case 'h':
			case 'H':
				print_instructions();
				break;
			case 'a':
			case 'A':
				mode=AXIS;
				break;
			case 'm':
			case 'M':
				mode=MOVE;
				num_index=0;
				number[0]='\0';
				break;
			case 'v':
			case 'V':
				mode=VEL;
				num_index=0;
				number[0]='\0';
				break;
			case 'r':
			case 'R':
				mode=IDLE;
				move(CMD_LINE,20);printw("*wait*");refresh();
				pi_move_off_limit_switch(current_axis);
				move(CMD_LINE,20);printw("      ");refresh();
				break;
			case 's':
			case 'S':
				mode=MOVEREL;
				num_index=0;
				number[0]='\0';
				break;
			case 'o':
			case 'O':
				mode=IDLE;
				pi_set_motor(current_axis,(pi_get_motor(current_axis)==PI_MOTOR_ON)?PI_MOTOR_OFF:PI_MOTOR_ON);
				break;
			case 'b':
			case 'B':
				mode=IDLE;
				pi_set_brake(current_axis,(pi_get_brake(current_axis)==PI_BRAKE_ON)?PI_BRAKE_OFF:PI_BRAKE_ON);
				break;
			case 't':
			case 'T':
				mode=IDLE;
				pi_setQMC(PI_SET_MOTOR_MODE,current_axis,pi_getQMC(PI_GET_MOTOR_MODE,current_axis)?0:1);
				break;
			case 'p':
			case 'P':
				mode=IDLE;
				pi_setQMC(PI_SET_OUTPUT_MODE,current_axis,1);
				break;
			case 'd':
			case 'D':
				mode=IDLE;
				pi_setQMC(PI_SET_OUTPUT_MODE,current_axis,0);
				break;
			case 'e':
			case 'E':
				mode=IDLE;
				pi_setQMC(PI_RESET_EVENT_STATUS,current_axis,~(PI_MOTION_ERROR|PI_MOTION_COMPLETE));
				pi_setQMC(PI_SET_MOTOR_MODE,current_axis,1);
				pi_move_abs(current_axis,pi_get_pos(current_axis));	

				break;
			case '0':
			case '1':
			case '2':
			case '3':
				if(mode==AXIS){
					current_axis=key-'0';
					mode=IDLE;
					}

			case '-':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				switch(mode){
					case MOVEREL:
					case MOVE:
					case VEL:	
						if(num_index<10){
							number[num_index++]=key;
							number[num_index]='\0';
							}
						}
				break;
			case ENTER:
				pos=atoi(number);
				number[0]='\0';
				switch(mode){
					case MOVEREL:
						pos=pi_get_pos(current_axis)+pos;
						pi_move_abs(current_axis,pos);
						break;
					case MOVE:
						pi_move_abs(current_axis,pos);
						break;
					case VEL:
						pi_setQMC(PI_SET_VELOCITY,current_axis,pos);
						break;
					}
				mode=IDLE;
				break;
				
			}
		}
		while(key!='q');
		
	move(21,20);
	cbreak();
	printw("Press any key to quit");
	while(getch()==ESCAPE)
		refresh();
	pi_closeboard();
	printw("OK");refresh();
	endwin();
	}

void print_instructions(void){
int line=5;
	clear();
	move(line++,3); printw("Commands:");
	move(line++,3); printw("For this help screen press 'h':");
	move(line++,3); printw("To set axis type a# where # is the axis number");
	move(line++,3); printw("To move abs type m#####<enter> where ##### is the position");
	move(line++,3); printw("To move rel type s#####<enter> where ##### is the shift");
	move(line++,3); printw("To set velocity  v#####<enter> where ##### is counts/cycle");
	move(line++,3); printw("To move off limits      r");
	move(line++,3); printw("To set motor Output     o");
	move(line++,3); printw("To enable brakes        b");
	move(line++,3); printw("To set motor PWM        p");
	move(line++,3); printw("To set motor DAC        d");
	move(line++,3); printw("To find center          C");
	move(line++,3); printw("To clear errors         e");

	move(line++,3); printw("To quit type q");
	move(20,20);
	printw("Press <space> to continue");
	while(getch()!=' ')
		refresh();
	erase();
	}

void print_cmd(int mode,int axis, char *number){
	move(CMD_LINE,0);
	printw("Command",axis);
	move(CMD_LINE,10);
	printw("Axis %d",axis);
	move(CMD_LINE,20);
	switch(mode){
		case MOVE:
		printw("Moveto");
		break;
		case MOVEREL:
		printw("Shift ");
		break;
		case VEL:
		printw("set V ");
		break;
		case IDLE:
		default:
		printw("      ");
		}
	move(CMD_LINE,30);
	printw("%s       ",number);
	}


void print_moving(){
int i;
	for(i=0;i<4;i++){
		if(pi_moving(i)){
			move(MOVING_LINE,3+20*i);
			printw("%c",moving[mp]);
			}
		else{
			move(MOVING_LINE,3+20*i);
			printw("O",moving[mp]);
			}
			
		}
	mp++;
	if(mp>=4)mp=0;
	}

void print_async_event(int){
int n=0;
struct pi_stage_event_t data[32];
	n=pi_read_async_events(data, 32);
	if(n==0) return;
	move(ASYNC_LINE,1);
	printw("Asynchronous event: index=%d, axis=%d, timestamp=%ud, event= %x     ",data[0].index,data[0].axis,data[0].ts_low,data[0].event);
	}
	

void print_status(int axis){
int line,col,activity,event,signal,vel,pos,poserr;
int actvel;

	activity=pi_get_activity_status(axis);
	event=pi_get_event_status(axis);
	signal=pi_get_signal_status(axis);
	vel=pi_getQMC(PI_GET_VELOCITY,axis);
	pos=pi_getQMC(PI_GET_ACTUAL_POSITION,axis);
	poserr=pi_getQMC(PI_GET_POSITION_ERROR,axis);
	actvel=pi_getQMC(PI_GET_ACTUAL_VELOCITY,axis);
	actvel=(int)((float)actvel*2000.0/65536.0);	
	
	line=3;
	col=20*axis;
	move(line++,col);
	printw("Axis %d",axis);
	move(line++,col);
        if(!(signal & PI_SIGNAL_POS_LIMIT))
		printw("On +ve");
	if(!(signal & PI_SIGNAL_NEG_LIMIT))
		printw("On -ve");
	if((signal & PI_SIGNAL_POS_LIMIT) && (signal & PI_SIGNAL_NEG_LIMIT))
		printw("Lim OK");
	move(line++,col);
	printw("V=%d    ",vel);
	move(line++,col);
	if(activity & PI_MAX_VELOCITY){
		printw("V=MAX    ");
		}
	else{
		printw("V=%d    ",actvel);
		}
	move(line++,col);
	printw("P=%d    ",pos);
	move(line++,col);
	printw("E=%d    ",poserr);
	move(line++,col);
	printw("BRK=%d",pi_get_brake(axis));
	move(line++,col);
	printw("AMP=%d",pi_get_motor(axis));
	move(line++,col);
	printw("MTR=%d",pi_getQMC(PI_GET_MOTOR_MODE,axis));
	move(line++,col);
	printw("ENC SRC=%d",pi_getQMC(PI_GET_ENCODER_SOURCE,axis));
	move(line++,col);
	printw("ENC IDX=%d",signal&PI_ENCODER_INDEX?1:0);
	move(line++,col);
	printw("ENC HME=%d",signal&PI_ENCODER_HOME?1:0);
	move(line++,col);
	printw("AB=%d%d",signal&PI_ENCODER_A?1:0,signal&PI_ENCODER_B?1:0);

	move(line++,col);
	switch(pi_getQMC(PI_GET_OUTPUT_MODE,axis)){
		case	0:
			printw("Analog");
			break;
		case	1:
			printw("PWM+/-");
			break;
		case	2:
			printw("PWM50%%");
			break;
		default:
			printw("Error ");
			}
	if(event & PI_MOTION_ERROR){
		move(line++,col);
		printw("M %x   ",event);
		move(line++,col);
		printw("Error ");
		}
	else{
		move(line++,col);
		printw("M %x   ",event);
		move(line++,col);
		printw("  Ok  ");
		}

	}










void init_curses(void){
        initscr();
        start_color();
        init_pair(1,COLOR_WHITE,COLOR_BLUE);
        init_pair(2,COLOR_BLUE,COLOR_WHITE);
        init_pair(3,COLOR_RED,COLOR_WHITE);
        curs_set(0);
        noecho();
        keypad(stdscr,TRUE);
	}
